import { html, ref, slotted } from '@microsoft/fast-element';
import { endSlotTemplate, Listbox, startSlotTemplate } from '@microsoft/fast-foundation';
const slotsTemplate = html `
  <slot
    ${slotted({
    filter: Listbox.slottedOptionFilter,
    flatten: true,
    property: 'slottedOptions',
})}
  ></slot>
`;
export const baseComboboxTemplate = (context, definition, slots) => html `
  <template
    class="${(x) => (x.open ? 'open' : '')} ${(x) => (x.disabled ? 'disabled' : '')} ${(x) => x.position}"
    ?open="${(x) => x.open}"
    @click="${(x, c) => x.clickHandler(c.event)}"
    @focusin="${(x, c) => x.focusinHandler(c.event)}"
    @focusout="${(x, c) => x.focusoutHandler(c.event)}"
    @keydown="${(x, c) => x.keydownHandler(c.event)}"
    @mousedown="${(x, c) => x.mousedownHandler(c.event)}"
    role="combobox"
  >
    <div class="control" part="control">
      ${startSlotTemplate(context, definition)}
      <slot name="control">
        <input
          aria-activedescendant="${(x) => x.ariaActiveDescendant}"
          aria-autocomplete="list"
          aria-haspopup="listbox"
          aria-expanded="${(x) => x.ariaExpanded}"
          aria-controls="${(x) => x.listboxId}"
          autocomplete="${(x) => x.autocomplete}"
          tabindex="${(x) => (!x.disabled ? '0' : null)}"
          class="selected-value"
          part="selected-value"
          placeholder="${(x) => x.placeholder}"
          type="text"
          ?disabled="${(x) => x.disabled}"
          :value="${(x) => x.value}"
          @input="${(x, c) => x.inputHandler(c.event)}"
          @keyup="${(x, c) => x.keyupHandler(c.event)}"
          ${ref('control')}
        />
        <div class="indicator" part="indicator" aria-hidden="true">
          <slot name="indicator">${definition.indicator || ''}</slot>
        </div>
      </slot>
      ${endSlotTemplate(context, definition)}
    </div>
    <div
      class="listbox"
      id="${(x) => x.listboxId}"
      part="listbox"
      role="listbox"
      ?disabled="${(x) => x.disabled}"
      ?hidden="${(x) => !x.open}"
      ${ref('listbox')}
    >
      ${slots}
    </div>
  </template>
`;
export const foundationBaseComboboxTemplate = (context, definition) => baseComboboxTemplate(context, definition, slotsTemplate);
