import { __decorate } from "tslib";
import { ResourceType } from '@genesislcap/foundation-comms';
import { attr, observable } from '@microsoft/fast-element';
import debounce from 'lodash.debounce';
import { OptionsDatasource } from '../options-datasource';
import { dataserverCriteriaBuilder, logger, requestServerCriteriaBuilder } from '../utils';
import { BaseCombobox, defaultComboboxConfig, foundationComboboxShadowOptions, } from './base.combobox';
import { foundationComboboxStyles as styles } from './combobox.styles';
import { foundationComboboxTemplate as template } from './combobox.template';
const DEBOUNCE_TIME = 500;
/**
 * @tagname %%prefix%%-combobox
 */
export class Combobox extends BaseCombobox {
    constructor() {
        super(...arguments);
        this.async = false;
        this.debounce = DEBOUNCE_TIME;
    }
    debounceChanged(oldValue, newValue) {
        this.debouncedInputHandler = debounce(this.asyncSetFilter, newValue);
    }
    asyncSetFilter() {
        const datasourceNode = this.slottedDatasource
            .assignedNodes()
            .filter((x) => x instanceof OptionsDatasource);
        if (datasourceNode.length > 0) {
            const resourceType = datasourceNode[0].datasource.resourceType;
            if (resourceType === ResourceType.DATASERVER) {
                dataserverCriteriaBuilder(datasourceNode[0], this.control.value);
            }
            if (resourceType === ResourceType.REQUEST_SERVER) {
                requestServerCriteriaBuilder(datasourceNode[0], this.control.value);
            }
        }
        else {
            logger.warn('Options-datasource is not connected. Please add a datasource to use this functionality.');
        }
    }
    slottedOptionsChanged(prev, next) {
        const shouldEmit = this.async ? true : false;
        super.slottedOptionsChanged(prev, next);
        this['updateValue'](shouldEmit);
    }
    inputHandler(e) {
        if (this.async) {
            this.debouncedInputHandler();
        }
        else {
            super.inputHandler(e);
        }
    }
}
__decorate([
    observable
], Combobox.prototype, "slottedDatasource", void 0);
__decorate([
    attr({ mode: 'boolean' })
], Combobox.prototype, "async", void 0);
__decorate([
    attr
], Combobox.prototype, "debounce", void 0);
/**
 * The Foundation Combo-box
 *
 * @public
 * @remarks
 * HTML Element: \<foundation-combobox\>
 */
export const foundationCombobox = Combobox.compose(Object.assign({ baseName: 'combobox', template,
    styles, shadowOptions: foundationComboboxShadowOptions }, defaultComboboxConfig));
// NOTE: We might want to rename this one to ConnectedCombobox or something similar so we can reuse the name for the "Base" combobox.
// That would be a breaking change.
